#include "gtkipcalc.h"

#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>

#include "callbacks.h"
#include "interface.h"

static void set_entry_ip(GtkEntry * entry, unsigned long ip);

void set_entry_ip(GtkEntry * entry, unsigned long ip)
{
  unsigned int a[4];
  gchar buf[16];

  a[3] = (ip & 0xFF000000UL) >> 24;
  a[2] = (ip & 0x00FF0000UL) >> 16;
  a[1] = (ip & 0x0000FF00UL) >> 8;
  a[0] = ip & 0x000000FFUL;

  snprintf(buf, 16, "%u.%u.%u.%u", a[3], a[2], a[1], a[0]);
  gtk_entry_set_text(entry, buf);
  return;
}

void on_type_toggled(GtkToggleButton * togglebutton, gpointer user_data)
{
  gtk_entry_set_editable(GTK_ENTRY(mask_entry), FALSE);
  gtk_entry_set_editable(GTK_ENTRY(bits_entry), FALSE);
  gtk_entry_set_editable(GTK_ENTRY(addrs_entry), FALSE);

  switch (*((char *)user_data)) {
  case 'M':
    gtk_entry_set_editable(GTK_ENTRY(mask_entry), TRUE);
    break;
  case 'B':
    gtk_entry_set_editable(GTK_ENTRY(bits_entry), TRUE);
    break;
  case 'A':
    gtk_entry_set_editable(GTK_ENTRY(addrs_entry), TRUE);
    break;
  }
}

void on_apply_button_clicked(GtkButton * button, gpointer user_data)
{
  unsigned int a[4];
  unsigned long net, mask, gw, broadcast, range_s, range_e, addrs, bits, i;
  gchar buf[16];

  if (sscanf(gtk_entry_get_text(GTK_ENTRY(net_entry)), "%u.%u.%u.%u", &a[3], &a[2], &a[1], &a[0]) != 4 || (a[3] | a[2] | a[1] | a[0]) > 255) {
    show_error_window(_("Invalid network"));
    return;
  }
  net = (a[3] << 24) + (a[2] << 16) + (a[1] << 8) + (a[0]);

  if (GTK_TOGGLE_BUTTON(mask_button)->active) {
    if (sscanf (gtk_entry_get_text(GTK_ENTRY(mask_entry)), "%u.%u.%u.%u", &a[3], &a[2], &a[1], &a[0]) != 4 || (a[3] | a[2] | a[1] | a[0]) > 255) {
      show_error_window(_("Invalid mask"));
      return;
    }
    mask = (a[3] << 24) + (a[2] << 16) + (a[1] << 8) + (a[0]);
  } else if (GTK_TOGGLE_BUTTON(bits_button)->active) {
    if (sscanf (gtk_entry_get_text(GTK_ENTRY(bits_entry)), "%u", &bits) != 1 || bits > 32) {
      show_error_window(_("Invalid bitmask"));
      return;
    }
    mask = 0xFFFFFFFFUL << (32 - bits);
  } else {
    if (sscanf(gtk_entry_get_text(GTK_ENTRY(addrs_entry)), "%lu", &addrs) != 1) {
      show_error_window(_("Invalid number of addresses"));
      return;
    }
    addrs += 1;
    for (i = 4; i < 0x80000000UL && ((addrs & (i - 1)) < addrs); i <<= 1);
    if (i == 0x80000000UL) {
      show_error_window(_("Invalid number of addresses"));
      return;
    }
    mask = ~(i - 1);
  }

  if ((net & mask) == 0) {
    show_error_window(_("Invalid combination of\nnetwork and netmask"));
    return;
  }

  net = net & mask;
  for (bits = 32; (mask & (1 << (32-bits))) == 0 /*bits > 0*/; bits--);
  addrs = ~mask - 1;
  gw = net + 1;
  broadcast = net | (~mask);
  range_s = gw;
  range_e = broadcast - 1;

  set_entry_ip(GTK_ENTRY(net_entry), net);
  set_entry_ip(GTK_ENTRY(mask_entry), mask);
  snprintf(buf, 16, "%lu", bits);
  gtk_entry_set_text(GTK_ENTRY(bits_entry), buf);
  snprintf(buf, 16, "%lu", addrs);
  gtk_entry_set_text(GTK_ENTRY(addrs_entry), buf);
  set_entry_ip(GTK_ENTRY(broadcast_entry), broadcast);
  set_entry_ip(GTK_ENTRY(gw_entry), gw);
  set_entry_ip(GTK_ENTRY(range_start_entry), range_s);
  set_entry_ip(GTK_ENTRY(range_end_entry), range_e);

  return;
}

